@@ -111,27 +111,11 @@
       psi.inv      <- psi.inv[,index, drop=FALSE]
       z            <- factor(z, labels=match(nn.ind, index))
       z            <- as.integer(levels(z))[z]
-
-    # Cluster Labels
-      psi          <- 1/psi.inv
-      sigma        <- if(uni) lapply(Gseq, function(g) as.matrix(psi[,g] + if(Q0) tcrossprod(as.matrix(lmat[,,g])) else 0L)) else lapply(Gseq, function(g) tcrossprod(lmat[,,g]) + diag(psi[,g]))
-      log.pis      <- log(pi.prop)
-      if(uni) {
-        log.probs  <- vapply(Gseq, function(g) stats::dnorm(data, mu[,g], sq_mat(sigma[[g]]), log=TRUE) + log.pis[g], numeric(N))
-      } else  {
-        log.probs  <- try(vapply(Gseq, function(g) dmvn(data, mu[,g], if(Q0) sigma[[g]] else sq_mat(sigma[[g]]), log=TRUE, isChol=!Q0) + log.pis[g], numeric(N)), silent=TRUE)
-        if(zerr    <- inherits(log.probs, "try-error")) {
-         log.probs <- vapply(Gseq, function(g) { sigma <- if(Q0) is.posi_def(sigma[[g]], make=TRUE)$X.new else sq_mat(sigma[[g]]); dmvn(data, mu[,g], sigma, log=TRUE, isChol=!Q0) + log.pis[g] }, numeric(N))
-        }
-      }
-      z            <- gumbel_max(probs=log.probs)
-      nn           <- tabulate(z, nbins=G)
-      nn0          <- nn > 0
-      nn.ind       <- which(nn0)
-      G.non        <- length(nn.ind)
-      dat.g        <- lapply(Gseq, function(g) data[z == g,, drop=FALSE])
+      nn           <- nn[index]
+      nn0          <- nn0[index]
 
     # Scores & Loadings
+      dat.g        <- lapply(Gseq, function(g) data[z == g,, drop=FALSE])
       c.data       <- lapply(Gseq, function(g) sweep(dat.g[[g]], 2L, mu[,g], FUN="-", check.margin=FALSE))
       if(Q0) {
         eta.tmp    <- lapply(Gseq, function(g) if(nn0[g]) .sim_score(N=nn[g], lmat=lmat[,,g], Q=Q, c.data=c.data[[g]], psi.inv=psi.inv[,g], Q1=Q1) else .empty_mat(nc=Q))
@@ -173,6 +157,24 @@
         }
       }
 
+    # Cluster Labels
+      psi          <- 1/psi.inv
+      sigma        <- if(uni) lapply(Gseq, function(g) as.matrix(psi[,g] + if(Q0) tcrossprod(as.matrix(lmat[,,g])) else 0L)) else lapply(Gseq, function(g) tcrossprod(lmat[,,g]) + diag(psi[,g]))
+      log.pis      <- log(pi.prop)
+      if(uni) {
+        log.probs  <- vapply(Gseq, function(g) stats::dnorm(data, mu[,g], sq_mat(sigma[[g]]), log=TRUE) + log.pis[g], numeric(N))
+      } else  {
+        log.probs  <- try(vapply(Gseq, function(g) dmvn(data, mu[,g], if(Q0) sigma[[g]] else sq_mat(sigma[[g]]), log=TRUE, isChol=!Q0) + log.pis[g], numeric(N)), silent=TRUE)
+        if(zerr    <- inherits(log.probs, "try-error")) {
+         log.probs <- vapply(Gseq, function(g) { sigma <- if(Q0) is.posi_def(sigma[[g]], make=TRUE)$X.new else sq_mat(sigma[[g]]); dmvn(data, mu[,g], sigma, log=TRUE, isChol=!Q0) + log.pis[g] }, numeric(N))
+        }
+      }
+      z            <- gumbel_max(probs=log.probs)
+      nn           <- tabulate(z, nbins=G)
+      nn0          <- nn > 0
+      nn.ind       <- which(nn0)
+      G.non        <- length(nn.ind)
+
       if(zerr && !err.z) {                                     cat("\n"); warning("\nAlgorithm may slow due to corrections for Choleski decompositions of non-positive-definite covariance matrices\n", call.=FALSE)
         err.z      <- TRUE
       }
@@ -182,7 +184,7 @@
         if(sw["mu.sw"])                mu.store[,,new.it]   <- mu
         if(all(sw["s.sw"], Q0))       eta.store[,,new.it]   <- eta
         if(all(sw["l.sw"], Q0))     load.store[,,,new.it]   <- lmat
-        if(sw["psi.sw"])              psi.store[,,new.it]   <- 1/psi.inv
+        if(sw["psi.sw"])              psi.store[,,new.it]   <- psi
         if(sw["pi.sw"])                 pi.store[,new.it]   <- pi.prop
         if(learn.alpha)               alpha.store[new.it]   <- pi.alpha
                                          z.store[new.it,]   <- as.integer(z)
