@@ -2,7 +2,7 @@
 #'
 #' Functions to simulate data of any size and dimension from a (infinite) mixture of (infinite) factor analysers parameterisation or fitted object.
 #' @param N,G,P Desired overall number of observations, number of clusters, and number of variables in the simulated data set. All must be a single integer.
-#' @param Q Desired number of cluster-specific latent factors in the simulated data set. Can be specified either as a single integer if all clusters are to have the same number of factors, or a vector of length \code{G}. Defaults to \code{floor(log(P))} in each cluster. Should be less than the associated \code{\link{Ledermann}} bound and the number of observations in the corresponding cluster. The argument \code{forceQ} can be used to enforce this upper limit.
+#' @param Q Desired number of cluster-specific latent factors in the simulated data set. Can be specified either as a single integer if all clusters are to have the same number of factors, or a vector of length \code{G}. Defaults to \code{floor(log(P))} in each cluster. Should be less than the associated \code{\link{Ledermann}} bound and the number of observations in the corresponding cluster. The argument \code{forceQg} can be used to enforce this upper limit.
 #' @param pis Mixing proportions of the clusters in the dataset if \code{G} > 1. Must sum to 1. Defaults to \code{rep(1/G, G)}.
 #' @param mu True values of the mean parameters, either as a single value, a vector of length \code{G}, a vector of length \code{P}, or a \code{G * P} matrix. If \code{mu} is missing, \code{loc.diff} is invoked to simulate distinct means for each cluster by default.
 #' @param psi True values of uniqueness parameters, either as a single value, a vector of length \code{G}, a vector of length \code{P}, or a \code{G * P} matrix. As such the user can specify uniquenesses as a diagonal or isotropic matrix, and further constrain uniquenesses across clusters if desired. If \code{psi} is missing, uniquenesses are simulated via \code{1/rgamma(P, 2, 1)} within each cluster by default.
@@ -11,11 +11,13 @@
 #' @param nn An alternative way to specify the size of each cluster, by giving the exact number of observations in each cluster explicitly. Must sum to \code{N}.
 #' @param loc.diff A parameter to control the closeness of the clusters in terms of the difference in their location vectors. Only relevant if \code{mu} is NOT supplied. Defaults to \code{2}.
 #'
-#' More specifically, \code{loc.diff} (if invoked) is invoked as follows: means are simulated with the vector of cluster-specific hypermeans given by \code{scale(1:G, center=TRUE, scale=FALSE) * loc.diff}.
+#' More specifically, \code{loc.diff} (if invoked) is invoked as follows: means are simulated with the vector of cluster-specific hypermeans given by:
+#'
+#'\code{scale(1:G, center=TRUE, scale=FALSE) * loc.diff}.
 #' @param non.zero Controls the number of non-zero entries in each loadings column (per cluster) \strong{only} when \code{loadings} is not explicitly supplied. Values must be integers in the interval \code{[1,P]}. Defaults to \code{P}. The positions of the zeros are randomised, and non-zero entries are drawn from a standard normal.
 #'
-#' Must be given as a list of length \code{G} of vectors of length corresponding to \code{Q} when \code{G>1}. Can be given either as such a list or simply a vector of length \code{Q} when \code{G=1}. Alternatively, a single integer can be supplied, common across all loadings columns across all clusters. In any case, \code{non.zero} will be affected by \code{forceQ=TRUE} by default (see below).
-#' @param forceQ A logical indicating whether the upper limit on the number of cluster-specific factors \code{Q} is enforced. Defaults to \code{TRUE} for \code{sim_IMIFA_data}, but is always \code{FALSE} for \code{sim_IMIFA_model}. Note that when \code{forceQ=TRUE} is invoked, \code{non.zero} (see above) is also affected.
+#' Must be given as a list of length \code{G} of vectors of length corresponding to \code{Q} when \code{G>1}. Can be given either as such a list or simply a vector of length \code{Q} when \code{G=1}. Alternatively, a single integer can be supplied, common across all loadings columns across all clusters. In any case, \code{non.zero} will be affected by \code{forceQg=TRUE} by default (see below).
+#' @param forceQg A logical indicating whether the upper limit on the number of cluster-specific factors \code{Q} is enforced. Defaults to \code{TRUE} for \code{sim_IMIFA_data}, but is always \code{FALSE} for \code{sim_IMIFA_model}. Note that when \code{forceQg=TRUE} is invoked, \code{non.zero} (see above) is also affected.
 #' @param method A switch indicating whether the mixture to be simulated from is the conditional distribution of the data given the latent variables (default), or simply the marginal distribution of the data.
 #' @param res An object of class "\code{Results_IMIFA}" generated by \code{\link{get_IMIFA_results}}.
 #'
@@ -25,7 +27,7 @@
 #'
 #' Missing values are not allowed in any of \code{pis}, \code{mu}, \code{psi}, \code{loadings}, \code{scores} & \code{nn}.
 #' @export
-#' @references Murphy, K., Gormley, I. C. and Viroli, C. (2018) Infinite Mixtures of Infinite Factor Analysers, \emph{to appear}. <\href{https://arxiv.org/abs/1701.07010v4}{arXiv:1701.07010v4}>.
+#' @references Murphy, K., Viroli, C., and Gormley, I. C. (2019) Infinite mixtures of infinite factor analysers, \emph{Bayesian Analysis}, 1-27. <\href{https://projecteuclid.org/euclid.ba/1570586978}{doi:10.1214/19-BA1179}>.
 #' @keywords utility
 #' @importFrom Rfast "is.symmetric"
 #' @name sim_IMIFA
@@ -45,7 +47,7 @@
 #'                nn = NULL,
 #'                loc.diff = 2,
 #'                non.zero = P,
-#'                forceQ = TRUE,
+#'                forceQg = TRUE,
 #'                method = c("conditional", "marginal"))
 #' @examples
 #' # Simulate 100 observations from 3 balanced clusters with cluster-specific numbers of latent factors
@@ -62,15 +64,15 @@
 #' # Examine the overlap with a pairs plot of 5 randomly chosen variables
 #' pairs(sim_data[,sample(1:20, 5)], col=labels)
 #'
-#' # Fit a MIFA model to this data
-#' # tmp    <- mcmc_IMIFA(sim_data, method="MIFA", range.G=3, n.iters=5000)
+#' \donttest{# Fit a MIFA model to this data
+#' # tmp    <- mcmc_IMIFA(sim_data, method="MIFA", range.G=3, n.iters=5000)}
 #' @seealso \code{\link{mcmc_IMIFA}} for fitting an IMIFA related model to the simulated data set.
 #'
 #' \code{\link{get_IMIFA_results}} for generating input for \code{sim_IMIFA_model}.
 #'
 #' \code{\link{Ledermann}} for details on the upper-bound for \code{Q}. Note that this function accounts for isotropic uniquenesses, if \code{psi} is supplied in that manner, in computing this bound.
 sim_IMIFA_data <- function(N = 300L, G = 3L, P = 50L, Q = rep(floor(log(P)), G), pis = rep(1/G, G), mu = NULL, psi = NULL, loadings = NULL,
-                           scores = NULL, nn = NULL, loc.diff = 2, non.zero = P, forceQ = TRUE, method = c("conditional", "marginal")) {
+                           scores = NULL, nn = NULL, loc.diff = 2, non.zero = P, forceQg = TRUE, method = c("conditional", "marginal")) {
 
   N            <- unname(as.integer(N))
   G            <- unname(as.integer(G))
@@ -101,7 +103,7 @@ sim_IMIFA_data <- function(N = 300L, G = 3L, P = 50L, Q = rep(floor(log(P)), G),
        anyNA(nn)  ||
        any(length(nn)    != G,
            sum(nn)       != N))           stop(paste0("'nn' must be an integer vector of length G=", G, " which sums to N=", N, " without missing values"), call.=FALSE)
-    if(any(nn  <= 0))                     stop("All 'nn' values must be strictly positive; simulating empty clusters not allowed", call.=FALSE)
+    if(any(nn  <= 0))                     warning("All 'nn' values should be strictly positive; are you sure you want to simulate empty clusters?\n",       call.=FALSE, immediate.=FALSE)
   } else {
     if(!is.numeric(pis)  ||
        anyNA(pis) ||
@@ -117,7 +119,7 @@ sim_IMIFA_data <- function(N = 300L, G = 3L, P = 50L, Q = rep(floor(log(P)), G),
       iter     <- iter    + 1L
       nn0      <- any(nn <= 0)
     }
-    if(nn0)                               stop("Simulating empty clusters not allowed; try supplying 'nn' directly instead of small 'pis'", call.=FALSE)
+    if(nn0)                               warning("Empty clusters generated; are you sure?\n Try supplying 'nn' directly instead of small 'pis\n'",         call.=FALSE, immediate.=FALSE)
   }
 
   if(!(mumiss  <- missing(mu)))       {
@@ -137,14 +139,14 @@ sim_IMIFA_data <- function(N = 300L, G = 3L, P = 50L, Q = rep(floor(log(P)), G),
        !is.numeric(scores))               stop("Invalid 'scores' supplied: must be a numeric matrix", call.=FALSE)
     if(anyNA(scores))                     stop("Missing values are not allowed in 'scores'", call.=FALSE)
   }
-  Q.warn       <- pmin(nn - 1L, Ledermann(P, isotropic=!psimiss && length(unique(psisup[,1L])) == 1))
+  Q.warn       <- pmin(pmax(0L, nn - 1L), Ledermann(P, isotropic=!psimiss && length(unique(psisup[,1L])) == 1))
   if(any(Q.ind <- Q > Q.warn))        {
-    if(length(forceQ)   != 1 ||
-       !is.logical(forceQ))               stop("'forceQ' must be a single logical indicator", call.=FALSE)
+    if(length(forceQg)  != 1 ||
+       !is.logical(forceQg))              stop("'forceQg' must be a single logical indicator", call.=FALSE)
     warn.Q     <- paste0("Too many factors generated relative to P=", P, " and the number of observations in cluster", ifelse(sum(Q.ind) > 1, "s ", " "), paste(which(Q.ind), paste0("(", nn[Q.ind], ")"), collapse=", "))
-    if(isTRUE(forceQ))   {                warning(paste0(warn.Q, "\nEnforced the corresponding upper-bound", ifelse(sum(Q.ind) > 1, "s ", " "), "{", paste(Q.warn[Q.ind], collapse=", "), "}\n"), call.=FALSE, immediate.=TRUE)
+    if(isTRUE(forceQg))  {                warning(paste0(warn.Q, "\nEnforced the corresponding upper-bound", ifelse(sum(Q.ind) > 1, "s ", " "), "{", paste(Q.warn[Q.ind], collapse=", "), "}\n"), call.=FALSE, immediate.=TRUE)
       Q[Q.ind] <- Q.warn[Q.ind]
-    } else                                warning(paste0(warn.Q, "\nConsider setting 'forceQ' to 'TRUE'"), call.=FALSE, immediate.=TRUE)
+    } else if(G > 1)                      warning(paste0(warn.Q, "\nConsider setting 'forceQg' to 'TRUE'"), call.=FALSE, immediate.=TRUE)
   }
   if(!(lammiss <- is.null(loadings))) {
     lmat       <- loadings
@@ -180,7 +182,7 @@ sim_IMIFA_data <- function(N = 300L, G = 3L, P = 50L, Q = rep(floor(log(P)), G),
       (is.list(non.zero)     &&
        length(non.zero)      != 1)))  {   stop(paste0("'non.zero' must be a list of length G=", G), call.=FALSE)
     } else non.zero   <- if(all(G == 1, length(non.zero) > 1)) list(non.zero) else as.list(non.zero)
-    if(forceQ && any(Q.ind))  {           message("Also forcing 'non-zero' to conform to 'Q'...\n")
+    if(forceQg && any(Q.ind)) {           message("Also forcing 'non-zero' to conform to 'Q'...\n")
       non.zero <- lapply(Gseq, function(g, ng=non.zero[[g]]) ng[seq_len(pmin(length(ng), Q.warn[g]))])
     }
     non.zero   <- lapply(non.zero, function(x) if(!is.null(x) && (length(x) > 1 || x != 0)) x)
@@ -198,7 +200,7 @@ sim_IMIFA_data <- function(N = 300L, G = 3L, P = 50L, Q = rep(floor(log(P)), G),
   sq_mat       <- if(P > 50)  function(x) diag(sqrt(diag(x))) else sqrt
 
 # Simulate true parameter values
-  true.zlab    <- factor(rep(Gseq, nn), labels=Gseq)
+  true.zlab    <- factor(rep(Gseq, nn), labels=Gseq[nn > 0])
   if(method    == "conditional")   {
     Q.max      <- max(Q)
     eta.true   <- if(smiss) .sim_eta_p(Q=Q.max, N=N) else scores
@@ -280,9 +282,10 @@ sim_IMIFA_data <- function(N = 300L, G = 3L, P = 50L, Q = rep(floor(log(P)), G),
 #' sim_IMIFA_model(res,
 #'                 method = c("conditional", "marginal"))
   sim_IMIFA_model      <- function(res, method = c("conditional", "marginal")) {
-    UseMethod("sim_IMIFA_model")
+      UseMethod("sim_IMIFA_model")
   }
 
+#' @method sim_IMIFA_model Results_IMIFA
 #' @export
   sim_IMIFA_model.Results_IMIFA   <-      function(res, method = c("conditional", "marginal")) {
     if(!all(is.character(method)))        stop("'method' must be a character vector of length 1", call.=FALSE)
@@ -306,5 +309,5 @@ sim_IMIFA_data <- function(N = 300L, G = 3L, P = 50L, Q = rep(floor(log(P)), G),
                      loadings = res$Loadings$post.load,
                      scores   = res$Scores$post.eta,
                      method   = meth,
-                     forceQ   = FALSE)))
+                     forceQg   = FALSE)))
   }
